@@ -1,6 +1,7 @@
 """
 Handles deploy logic for components
 """
+import copy
 import importlib
 import json
 import logging
@@ -9,10 +10,10 @@ import sys
 import yaml
 
 from .config import merge_cfgs
-from .images import import_images
-from .utils import cancel_builds, load_cfg_file, oc, wait_for_ready_threaded
+from .images import get_is_configs, import_images
+from .utils import load_cfg_file, oc, trigger_builds, wait_for_ready_threaded
 from .secrets import import_secrets, SecretImporter
-from .templates import get_templates_in_dir
+from .templates import Template, get_templates_in_dir
 
 
 log = logging.getLogger(__name__)
@@ -80,17 +81,15 @@ def deploy_components(
         oc("apply", "-f", "-", "-n", project_name, _in=template.dump_processed_json())
 
         # Mark certain resources in this component as ones we need to wait on
-        for restype in ("dc", "bc", "sts"):
+        for restype in ("dc", "sts"):
             resources_to_wait_for.extend(
                 [(restype, name) for name in template.get_processed_names_for_restype(restype)]
             )
 
         # Re-trigger any builds for deployed build configs
-        bcs = template.get_processed_names_for_restype("bc")
-        for name in bcs:
-            log.info("Re-triggering builds for '%s'", name)
-            cancel_builds(name)
-            oc("start-build", "bc/{}".format(name))
+        bcs = template.get_processed_items_for_restype("bc")
+        if bcs:
+            resources_to_wait_for.extend(trigger_builds(bcs))
 
     # Wait on all resources that have been marked as 'resources to wait for'
     if wait:
@@ -99,15 +98,14 @@ def deploy_components(
     return processed_templates_by_name
 
 
-def deploy_dry_run(
+def _deploy_dry_run(
     project_name,
     template_dir,
     components,
     variables_per_component,
-    wait=False,
-    timeout=None,
-    resources_scale_factor=1.0,
-    label=None,
+    label,
+    resources_scale_factor,
+    jinja_only,
 ):
     """
     Similar to deploy_components, but only processes the templates.
@@ -124,9 +122,18 @@ def deploy_dry_run(
             )
 
         template = templates_by_name.get(comp_name)
-        template.process(variables_per_component.get(comp_name, {}), resources_scale_factor, label)
+        if jinja_only:
+            template.process_jinja(variables_per_component.get(comp_name, {}))
 
-        if not template.processed_content:
+        else:
+            template.process(
+                variables_per_component.get(comp_name, {}), resources_scale_factor, label,
+            )
+
+        content_attr_name = "processed_content"
+        if jinja_only:
+            content_attr_name = "processed_jinja_content"
+        if not getattr(template, content_attr_name):
             log.info("Component %s has an empty template, skipping...", comp_name)
             continue
 
@@ -135,6 +142,48 @@ def deploy_dry_run(
     return processed_templates_by_name
 
 
+def deploy_dry_run(
+    project_name,
+    template_dir,
+    components,
+    variables_per_component,
+    wait=False,
+    timeout=None,
+    resources_scale_factor=1.0,
+    label=None,
+):
+    return _deploy_dry_run(
+        project_name,
+        template_dir,
+        components,
+        variables_per_component,
+        label,
+        resources_scale_factor,
+        jinja_only=False,
+    )
+
+
+def deploy_dry_run_jinja_only(
+    project_name,
+    template_dir,
+    components,
+    variables_per_component,
+    wait=False,
+    timeout=None,
+    resources_scale_factor=1.0,
+    label=None,
+):
+    return _deploy_dry_run(
+        project_name,
+        template_dir,
+        components,
+        variables_per_component,
+        label,
+        resources_scale_factor,
+        jinja_only=True,
+    )
+
+
 DEFAULT_DEPLOY_METHODS = (None, deploy_components, None)
 
 
@@ -153,7 +202,7 @@ def _load_module(path, service_set):
     return module
 
 
-def _get_custom_methods(service_set, service_set_dir, root_custom_dir):
+def _get_custom_deploy_methods(service_set, service_set_dir, root_custom_dir):
     """
     Look for custom deploy module and import its methods.
     """
@@ -164,9 +213,7 @@ def _get_custom_methods(service_set, service_set_dir, root_custom_dir):
         )
 
     if not module:
-        module = _load_module(
-            os.path.join(root_custom_dir, f"deploy.py"), service_set
-        )
+        module = _load_module(os.path.join(root_custom_dir, f"deploy.py"), service_set)
 
     if not module:
         log.exception("Error loading custom deploy script, using default deploy methods")
@@ -205,7 +252,7 @@ def _get_custom_methods(service_set, service_set_dir, root_custom_dir):
 
 def _get_deploy_methods(config, service_set_name, service_set_dir, root_custom_dir):
     if config.get("custom_deploy_logic", False):
-        pre_deploy_method, deploy_method, post_deploy_method = _get_custom_methods(
+        pre_deploy_method, deploy_method, post_deploy_method = _get_custom_deploy_methods(
             service_set_name, service_set_dir, root_custom_dir
         )
     else:
@@ -213,10 +260,14 @@ def _get_deploy_methods(config, service_set_name, service_set_dir, root_custom_d
     return pre_deploy_method, deploy_method, post_deploy_method
 
 
-def generate_dry_run_content(all_processed_templates, output="yaml", to_dir=None):
+def generate_dry_run_content(all_processed_templates, output="yaml", to_dir=None, jinja_only=False):
     """
     Write processed template content to output directory, or print to stdout if no dir given.
     """
+    content_attr_name = "processed_content"
+    if jinja_only:
+        content_attr_name = "processed_jinja_content"
+
     if to_dir:
         to_dir = os.path.abspath(to_dir)
         try:
@@ -228,15 +279,22 @@ def generate_dry_run_content(all_processed_templates, output="yaml", to_dir=None
 
     for service_set, processed_templates in all_processed_templates.items():
         for template_name, template_obj in processed_templates.items():
-            if not template_obj.processed_content:
+            # Check if the template_obj is an actual template... the "_imagestreams" component
+            # added during dry run is just a plain dict
+            if isinstance(template_obj, Template):
+                content = getattr(template_obj, content_attr_name)
+            else:
+                content = template_obj
+
+            if not content:
                 log.warning("Template '%s' had no processed content", template_name)
             else:
                 if output not in ["yaml", "json"]:
                     output = "yaml"
                 if output == "yaml":
-                    text = yaml.dump(template_obj.processed_content, default_flow_style=False)
+                    text = yaml.dump(content, default_flow_style=False)
                 else:
-                    text = json.dumps(template_obj.processed_content, indent=2)
+                    text = json.dumps(content, indent=2)
 
                 if to_dir:
                     service_set_dir = os.path.join(to_dir, service_set)
@@ -245,7 +303,7 @@ def generate_dry_run_content(all_processed_templates, output="yaml", to_dir=None
                     with open(file_path, "w") as f:
                         f.write(text)
                 else:
-                    print("\n# {}/{}".format(service_set, template_name))
+                    print("---\n# {}/{}".format(service_set, template_name))
                     print(text)
 
 
@@ -279,6 +337,7 @@ class DeployRunner(object):
         self.dry_run = dry_run
         self.dry_run_opts = dry_run_opts or {}
         self.env_config_handler = env_config_handler
+        self._base_is_configs = {}
 
     def _get_variables(self, service_set_name, service_set_dir, component):
         variables = {}
@@ -434,7 +493,26 @@ class DeployRunner(object):
 
         if self.dry_run:
             log.info("Doing a DRY RUN of deployment")
-            pre_deploy_func, deploy_func, post_deploy_func = None, deploy_dry_run, None
+            jinja_only = self.dry_run_opts.get("jinja_only")
+            deploy_dry_run_func = deploy_dry_run_jinja_only if jinja_only else deploy_dry_run
+            pre_deploy_func, deploy_func, post_deploy_func = None, deploy_dry_run_func, None
+
+            is_configs = copy.deepcopy(self._base_is_configs)
+            new_is_configs = get_is_configs(set_cfg, self.env_config_handler.env_names)
+            for istag, is_config in new_is_configs.items():
+                # If an istag is imported by the base _cfg, due to the way ImageImporter works,
+                # it will not be re-imported again. So don't overwrite that istag in our
+                # processed output either.
+                if istag not in is_configs:
+                    is_configs[istag] = is_config
+
+            if is_configs:
+                processed_templates["_imagestreams"] = {
+                    "apiVersion": "v1",
+                    "kind": "List",
+                    "items": [config for _, config in is_configs.items()],
+                }
+
         else:
             import_secrets(set_cfg, self.env_config_handler.env_names)
             import_images(set_cfg, self.env_config_handler.env_names)
@@ -493,6 +571,7 @@ class DeployRunner(object):
         base_cfg = self._get_base_cfg()
         deploy_order = base_cfg.get("deploy_order", {})
 
+        self._base_is_configs = get_is_configs(base_cfg, self.env_config_handler.env_names)
         if not self.dry_run:
             import_secrets(base_cfg, self.env_config_handler.env_names)
             import_images(base_cfg, self.env_config_handler.env_names)
