@@ -143,6 +143,7 @@ class Template(object):
         self.path = path
         self.file_name, self.file_extension = os.path.splitext(self.path)
         self.processed_content = {}
+        self.processed_jinja_content = {}
 
     @staticmethod
     def _jinja_safe(data):
@@ -246,8 +247,13 @@ class Template(object):
         rendered_txt = template.render(**variables)
         if not rendered_txt.strip():
             log.info("Template '%s' is empty after jinja2 processing", self.file_name)
-            return None
-        return self._load_content(rendered_txt)
+            self.processed_jinja_content = {}
+        else:
+            self.processed_jinja_content = self._load_content(rendered_txt)
+        return self.processed_jinja_content
+
+    def process_jinja(self, variables):
+        return self._process_via_jinja2(variables)
 
     def process(self, variables, resources_scale_factor=1.0, label=None):
         # Run the template through jinja processing first
@@ -274,6 +280,19 @@ class Template(object):
     def dump_processed_json(self):
         return json.dumps(self.processed_content)
 
+    def get_processed_items_for_restype(self, restype):
+        """
+        Return list of objects of type 'restype' in the processed template
+
+        Note at the moment this only searches the 1st level of objects (for example,
+        it will not return the name of a pod embedded within a deployment config)
+        """
+        restype = parse_restype(restype)
+
+        return [
+            obj for obj in self.processed_content.get("items", []) if obj["kind"].lower() == restype
+        ]
+
     def get_processed_names_for_restype(self, restype):
         """
         Return list of names for all objects of type 'restype' in the processed template
@@ -281,12 +300,4 @@ class Template(object):
         Note at the moment this only searches the 1st level of objects (for example,
         it will not return the name of a pod embedded within a deployment config)
         """
-        restype = parse_restype(restype)
-
-        names = []
-
-        for obj in self.processed_content.get("items", []):
-            if obj["kind"].lower() == restype:
-                names.append(obj["metadata"]["name"])
-
-        return names
+        return [obj["metadata"]["name"] for obj in self.get_processed_items_for_restype(restype)]
