@@ -10,6 +10,7 @@ import os
 import yaml
 from functools import reduce
 
+from anytree import Node, RenderTree, PreOrderIter
 import sh
 from sh import ErrorReturnCode
 from wait_for import wait_for, TimedOutError
@@ -77,6 +78,19 @@ def object_merge(old, new, merge_lists=True):
     return new
 
 
+def traverse_keys(d, keys, default=None):
+    """
+    Allows you to look up a 'path' of keys in nested dicts without knowing whether each key exists
+    """
+    key = keys.pop(0)
+    item = d.get(key, default)
+    if len(keys) == 0:
+        return item
+    if not item:
+        return default
+    return traverse_keys(item, keys, default)
+
+
 def parse_restype(string):
     """
     Given a resource type or its shortcut, return the full resource type name.
@@ -163,15 +177,11 @@ def all_sets(template_dir):
 
 
 def _only_immutable_errors(err_lines):
-    return all(
-        "field is immutable after creation" in line.lower() for line in err_lines
-    )
+    return all("field is immutable after creation" in line.lower() for line in err_lines)
 
 
 def _conflicts_found(err_lines):
-    return any(
-        "error from server (conflict)" in line.lower() for line in err_lines
-    )
+    return any("error from server (conflict)" in line.lower() for line in err_lines)
 
 
 def _get_logging_args(args, kwargs):
@@ -278,7 +288,8 @@ def oc(*args, **kwargs):
             log.error("Command failed!  Aborting.")
             sys.exit(1)
         else:
-            log.warning("Non-zero return code ignored")
+            if not kwargs.get("_silent"):
+                log.warning("Non-zero return code ignored")
 
 
 def switch_to_project(project):
@@ -623,17 +634,23 @@ def get_server_info():
 
 
 def cancel_builds(bc_name):
-    oc("cancel-build", f"bc/{bc_name}", state="pending,running", _exit_on_err=False)
+    oc(
+        "cancel-build",
+        f"bc/{bc_name}",
+        state="new,pending,running",
+        _timeout=120,
+        _exit_on_err=False,
+    )
 
     # Check if there's any lingering builds
     builds = get_json("build", label=f"openshift.io/build-config.name={bc_name}")
     lingering_builds = []
-    for build in builds.get('items', []):
+    for build in builds.get("items", []):
         # delete these builds rather than cancelling them, since jenkins pipeline builds
         # can remain stuck in certain states if OpenShift Sync plugin is broken
-        status = build.get('status') or {}
-        phase = status.get('phase', "").lower()
-        if phase in ["new", "pending", "running"]:
+        status = build.get("status") or {}
+        phase = status.get("phase", "").lower()
+        if phase in ["new", "pending"]:
             build_name = build["metadata"]["name"]
             lingering_builds.append(build_name)
 
@@ -641,3 +658,115 @@ def cancel_builds(bc_name):
         log.warning("Found lingering builds for bc/%s which will be deleted", bc_name)
         for build_name in lingering_builds:
             oc("delete", "build", build_name)
+
+
+def get_input_image(buildconfig, trigger):
+    """
+    Look up the image stream input image for a build triggered by imagechange.
+    """
+    bc = buildconfig
+    input_image = None
+
+    if trigger.get("imageChange", {}) != {}:
+        # the image used for trigger is explicitly defined, we're done
+        input_image = trigger["imageChange"]["from"]["name"]
+        return input_image
+
+    # we need to look up the image used for trigger in the bc's configuration
+    # check if there's a dockerfile with FROM line
+    dockerfile = traverse_keys(bc, ["spec", "source", "dockerfile"])
+    if dockerfile:
+        for line in dockerfile.splitlines():
+            if line.startswith("FROM"):
+                input_image = line.split()[1]
+                if ":" not in input_image:
+                    input_image = f"{input_image}:latest"
+                break
+
+    # check if the source imagestreamtag is defined in the strategy config
+    for key in ["dockerStrategy", "sourceStrategy", "customStrategy"]:
+        from_kind = traverse_keys(bc, ["spec", "strategy", key, "from", "kind"], "").lower()
+        if from_kind == "imagestreamtag":
+            input_image = bc["spec"]["strategy"][key]["from"]["name"]
+
+    return input_image
+
+
+def get_build_tree(buildconfigs):
+    """
+    Analyze build configurations to find which builds are 'linked'.
+
+    Linked builds are those which output to an ImageStream that another BuildConfig then
+    uses as its 'from' image.
+
+    Returns a list of lists where item 0 in each list is the parent build and the items following
+    it are all child build configs that will be fired at some point after the parent completes
+    """
+    bcs_using_input_image = {}
+    bc_creating_output_image = {None: None}
+    node_for_bc = {}
+    for bc in buildconfigs:
+        bc_name = bc["metadata"]["name"]
+        node_for_bc[bc_name] = Node(bc_name)
+
+        # look up output image
+        if traverse_keys(bc, ["spec", "output", "to", "kind"], "").lower() == "imagestreamtag":
+            output_image = bc["spec"]["output"]["to"]["name"]
+            bc_creating_output_image[output_image] = bc_name
+
+        # look up input image
+        for trigger in traverse_keys(bc, ["spec", "triggers"], []):
+            if trigger.get("type", "").lower() == "imagechange":
+                input_image = get_input_image(bc, trigger)
+                if input_image not in bcs_using_input_image:
+                    bcs_using_input_image[input_image] = []
+                bcs_using_input_image[input_image].append(bc_name)
+
+    # attach each build to its parent build
+    for input_image, bc_names in bcs_using_input_image.items():
+        for bc_name in bc_names:
+            parent_bc = bc_creating_output_image.get(input_image)
+            if parent_bc:
+                node_for_bc[bc_name].parent = node_for_bc[parent_bc]
+
+    rendered_trees = []
+    root_nodes = [n for _, n in node_for_bc.items() if n.is_root]
+    for root_node in root_nodes:
+        for pre, _, node in RenderTree(root_node):
+            rendered_trees.append(f"  {pre}{node.name}")
+
+    if rendered_trees:
+        log.info("build config tree:\n\n%s", "\n".join(rendered_trees))
+
+    return [[node.name for node in PreOrderIter(root_node)] for root_node in root_nodes]
+
+
+def get_next_build(bc_name):
+    """
+    Return the upcoming build name we can expect to see for a given build config
+    """
+    json_data = get_json("bc", bc_name)
+    last_version = traverse_keys(json_data, ["status", "lastVersion"], 0)
+    next_build = "{}-{}".format(bc_name, last_version + 1)
+    return next_build
+
+
+def trigger_builds(buildconfigs):
+    """
+    Trigger parent build configs based on a build tree and return the resources to wait for
+    """
+    bcs = buildconfigs
+
+    builds_to_wait_for = []
+
+    build_tree = get_build_tree(bcs)
+    for sub_tree in build_tree:
+        parent = sub_tree[0]
+        for bc_name in sub_tree:
+            # Cancel any new/pending builds
+            cancel_builds(bc_name)
+            builds_to_wait_for.append(("build", get_next_build(bc_name)))
+        log.info("triggering build for '%s'", parent)
+        oc("start-build", "bc/{}".format(parent))
+
+    return builds_to_wait_for
